/*-
 * SPDX-License-Identifier: LGPL-2.1-only
 * Copyright 2022 UPLEX Nils Goroll Systemoptimierung. All rights reserved.
 * Author: Nils Goroll <nils.goroll@uplex.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA Also add information on how to contact you by
 * electronic and paper mail.
 */

#include "config.h"

#include <assert.h>
#include <string.h>
#include <limits.h>
#include <stdint.h>
#include <float.h>
#include <stdio.h>

#include "miniobj.h"

#include "fellow_tune.h"
#include "buddy_util.h"

/* initialize with sane defaults */
const char *
stvfe_tune_init(struct stvfe_tune *tune, size_t memsz, size_t dsksz,
    size_t objsize_hint)
{
	INIT_OBJ(tune, STVFE_TUNE_MAGIC);

//lint -save -e773 -e835
#define TUNE(t, n, d, min, max)			\
	tune->n = (d)
#include "tbl/fellow_tunables.h"
//lint -restore
	tune->memsz = memsz;
	tune->dsksz = dsksz;
	tune->objsize_hint = objsize_hint;
	return (stvfe_tune_check(tune));
}

// XXX VSL for warnigs

const char *
stvfe_tune_check(struct stvfe_tune *tune)
{
	unsigned l;
	size_t sz;

	assert (tune->dsksz >= tune->memsz);

#ifdef DOES_THIS_MAKE_SENSE
	/* the average object could be quite small, but larger chunks
	 * could be required, in which case it makes sense for the LRU
	 * to keep the larger chunks in the reserve
	 */
	l = log2up(tune->objsize_hint);
	if (tune->chunk_exponent > l) {
		fprintf(stderr, "fellow: chunk_exponent limited to %u "
		    "(next pow2 of objsize_hint)\n", l);
		tune->mem_reserve_chunks <<= (tune->chunk_exponent - l);
		tune->dsk_reserve_chunks <<= (tune->chunk_exponent - l);
		tune->chunk_exponent = l;
	}
#endif
	l = log2down(tune->memsz) - 4;
	if (tune->chunk_exponent > l) {
		fprintf(stderr,"fellow: chunk_exponent limited to %u "
		    "(less than 1/16 of memory size)\n", l);
		tune->mem_reserve_chunks <<= (tune->chunk_exponent - l);
		tune->dsk_reserve_chunks <<= (tune->chunk_exponent - l);
		tune->chunk_exponent = l;
	}

	sz = tune->memsz >> (tune->chunk_exponent - 1);
	assert(sz <= UINT_MAX);
	l = (unsigned)sz;
	if (tune->mem_reserve_chunks > l) {
		fprintf(stderr,"fellow: mem_reserve_chunks limited to %u "
		    "(less than 1/8 of memory size)\n", l);
		tune->mem_reserve_chunks = l;
	}

//lint --e{685,568} misc const comparisons
#define TUNE(t, n, d, min, max)						\
	if (tune->n < (min))						\
		return ("Value of " #n " is too small, minimum is " #min); \
	if (tune->n > (max))						\
		return ("Value of " #n " is too big, maximum is " #max); \

//lint -save -e773 -e835
#include "tbl/fellow_tunables.h"
//lint -restore
	return (NULL);
}
